#coding:utf8
"""
总结：
1. 变量形式定义的token，正则表达式的长度越长，优先级越高
2. 函数形式定义的token，优先级根据函数的定义顺序由上至下
"""
import os, sys
import re
import ply.lex as lex

literals = [',', '.', '(', ')', ';', '=']

reserved = [
	'CREATE','TABLE','UNSIGNED','NOT','NULL','DEFAULT','COMMENT','AUTO_INCREMENT','PRIMARY','UNIQUE','KEY','ENGINE',
	'CHARSET','CHARACTER','SET','USING','COLLATE','ROW_FORMAT', 'CURRENT_TIMESTAMP', 'ON', 'UPDATE'
]

tokens = [
	'iKEYWORD',
	'iINTEGER',
	'iFLOAT',

	'iCHAR', 
	'iESCAPE',
	'iLBRACE',
	'iRBRACE'
] + reserved

states = (
	('str', 'inclusive'),
	('name', 'inclusive'),
)

t_ignore = " \t\r"
t_ignore_comment_sig = r'--(\s[^\n]*)??\n'
t_ignore_comment_mul = r'\/\*(.|\n)+?\*\/'
t_str_ignore = ''
t_name_ignore = ''

t_iINTEGER = r'\d+'
t_iFLOAT = r'\d+\.\d+'

def t_iKEYWORD(t):
	r'[a-zA-Z][a-zA-Z0-9_]+'
	
	if t.value.upper() in reserved:
		t.type = t.value.upper()
	else:
		t.type = 'iKEYWORD'

	return t

########### 单引号的字符串

def t_begin_str(t):
	r'\''
	t.lexer.push_state('str')
	t.type = 'iLBRACE'
	return t

def t_str_escape(t):
	r'\'\''
	t.type = 'iESCAPE'
	return t

def t_str_iCHAR(t):
	r'[^\']'
	return t

def t_str_end(t):
	r'\''
	t.lexer.pop_state()
	t.type = 'iRBRACE'
	return t
	
########### 反引号的对象名称

def t_begin_name(t):
	r'`'
	t.lexer.push_state('name')
	t.type = 'iLBRACE'
	return t

def t_name_escape(t):
	r'``'
	t.value = '`'
	t.type = 'iESCAPE'
	return t

def t_name_iCHAR(t):
	r'[^`]'
	return t

def t_name_end(t):
	r'`'
	t.lexer.pop_state()
	t.type = 'iRBRACE'
	return t

def t_newline(t):
	r'\n'
	t.lexer.lineno += 1

########### 错误处理
	 
def t_error(t):
	raise SyntaxError(t)

lexer = lex.lex(debug=False)




import os
import sys


def get_program_directory():
    app_dir = os.getcwd()
    if not app_dir:
        cmd = sys.argv[0]
        app_dir = None
        if cmd:
            app_dir, filename = os.path.split(cmd)

    return os.path.abspath(app_dir)


def check_path(path):
    if not os.path.isabs(path):
        path = os.path.join(get_program_directory(), path)
    return path


def get_current_path(current_path, path):
    if not os.path.isabs(path):
        path = os.path.join(os.path.dirname(os.path.realpath(current_path)), path)
    return path



from faker.providers.date_time import Provider as DataTimeProvider
from datetime import timedelta, datetime
from dateutil.tz import tzutc
import random
import time


class Provider(DataTimeProvider):

    def __init__(self, generator, offset=0):
        super().__init__(generator)
        self.offset = offset

    def date_time_between(self, start_date=None, end_date=None, tzinfo=None, offset=None, format=None):
        """
        指定开始时间点，与过去时间，计算结果时间，按照指定的格式返回
        :param start_date: 随机时间范围的开始时间
        :param end_date: 随机时间范围的结束时间
        :param tzinfo: 时区设置
        :param offset: 时间偏移，示例： -2d，指的是在开始时间和结束时间中确定了某个时间点后再往前推2天。
        :param format: 返回的时间格式
        :return:
        """
        start_date = self._parse_date_time(start_date, tzinfo=tzinfo)
        end_date = self._parse_date_time(end_date, tzinfo=tzinfo)
        if not offset:
            offset = self.offset * 24 * 60 * 60
        else:
            offset = int(offset) * 24 * 60 * 60

        if end_date - start_date <= 1:
            ts = start_date + self.generator.random.random() - offset
        else:
            ts = self.generator.random.randint(start_date, end_date) - offset

        if tzinfo is None:
            result = datetime(1970, 1, 1, tzinfo=tzinfo) + timedelta(seconds=ts)
        else:
            result = (
                    datetime(1970, 1, 1, tzinfo=tzutc()) + timedelta(seconds=ts)
            ).astimezone(tzinfo)
        if format:
            result = result.strftime(format)
        return result

    def date_time_subtraction(self, start_date, sub_time, format=None, approximate_time=True):
        """
        指定开始时间点，与过去时间，计算结果时间，按照指定的格式返回
        :param start_date:
        :param sub_time:
        :param format:
        :param approximate_time: 是否返回浮动的时间（为true时将对sub_time时间进行随机正负一倍以内来返回）
        :return:
        """
        if isinstance(start_date, str) and format:
            start_date = datetime.strptime(start_date, format)
        elif isinstance(start_date, int):
            start_date = datetime.utcfromtimestamp(start_date)
        sub_time = self._parse_date_string(sub_time)
        key, value = list(sub_time.items())[0]
        appr_value = value + value * random.uniform(-1, 1)
        if approximate_time:
            sub_time.update({key: appr_value})
        sub_time1 = timedelta(**sub_time)
        end_date = start_date + sub_time1
        if format:
            return end_date.strftime(format)
        else:
            return end_date.second

    def now(self, format=None):
        """
        返回当前时间，format参数存在时按照format参数指定的时间格式进行返回，
        否则返回时间戳
        :param format: 格式化时间格式 %Y-%m-%d %H:%M:%S
        :return:
        """
        if format:
            r = datetime.now().strftime(format)
        else:
            r = int(time.time() * 1000)
        return r



import yaml
from dbfaker.common.database import Database
import sys
import os
from dbfaker.utils.constant import __version__
import argparse


def table_name_to_table_building_statement(db_session, tables):
    table_words = ';\n'.join([db_session.query_table(s) for s in tables]) + ';'
    print(table_words)
    return table_words


def start(connect, table_names=None,
          sql_file=None, output=None, **kwargs):
    from dbfaker.nsqlparse.mysql_create_table_yacc import parser
    if table_names:
        if "," in table_names:
            tables = table_names.split(",")
        else:
            tables = [table_names]
        file_name = '_'.join(tables) + '_meta.yml'
        session = Database(connect)
        table_building_statement = table_name_to_table_building_statement(session, tables)
    else:
        file_name = os.path.split(sql_file)[1] + '_meta.yml'
        with open(sql_file, encoding='utf-8')as f:
            table_building_statement = f.read()

    result = {
        "package": [],
        "env": {},
        'tables': [],
        "extraction": {}

    }

    r = parser.parse(table_building_statement)

    for i in r:
        table_obj = {'table': i.get("table"), 'comment': i.get("comment"), "columns": []}
        for j in i['columns']:
            table_obj['columns'].append(
                {"column": j.get("column"), 'comment': j.get("comment"), 'engine': None, 'rule': None}
            )
        result['tables'].append(table_obj)
    if not output:
        output = file_name
        if os.path.exists('data'):
            output = os.path.join('data', output)
    f = open(output, 'w', encoding='utf-8')
    f.write('''# 请完善此文件中每个字段的生成规则
# 规则说明：
# package: 动态导包，在下方字段使用了jinja2模板且在模板语法中使用了非Python基础库时需要在此动态声明导入需要的包;使用示例:
# package:
#  - datetime  # 引入datetime包，可在后续的jinja2模板中使用
#  - os

# env: 可在此处预生成环境变量，给下方字段生成时引用；描述方式如下：
# env:
#  name:  # 全局变量名称
#    engine: eq  # 生成规则方法，与下面的字段生成方法一样
#    rule:  # "engine"方法中接收的参数
#      value: 'test'  

# tables： 该字段描述了表字段的生成规则，需要填写数据库字段中的engine与rule字段，为空时数据库字段也为空；示例(给数据库中t_sys_user表中age字段生成从40到80的随机数)：
# tables:
# - table: t_sys_user
#   comment: 用户表
#   columns:
#   - column: age  # 数据库中字段名
#     comment: '年龄'  # 字段备注信息
#     engine: randint  # 生成字段值调用的方法，必须是faker库中存在或者自行注册到faker库中的方法
#     rule: 
#        value: [40, 80]  # 上述方法中接收到的参数
       
# extraction： 该字段描述了需要从生成字段中提取哪些变量来返回，写自动化测试用例时可能会用到；举例：
# extraction：
#   user_name:
#     value: '{{ t_sys_user.name }}'  # 返回上面生成的用户姓名
#     default: '测试用户'  # 可指定默认值，在上述字段不存在或者为空时返回默认值
#   user_id:
#     value: '{{ t_sys_user.id }}'  


''' + yaml.dump(result, encoding='utf-8', allow_unicode=True, default_flow_style=False, sort_keys=False).decode())
    f.close()
    print('table转ymal文件成功，文件路径：{}'.format(os.path.abspath(output)))


def parse_args():
    if '--version' in sys.argv:
        print(__version__)
        exit(0)

    parser = argparse.ArgumentParser(
        description='数据库表转数据生成yaml文件格式工具')
    parser.add_argument('type', nargs='?', action='store', default='table_name',
                        help='数据来源，table_name： 通过输入表名与数据库链接方式，在数据库中获取数据库建表语句；\n table_statement: 指定数据库建表语句的sql文件路径')
    parser.add_argument('--connect', nargs='?', action='store',
                        help='数据库连接语法，例如：mysql+mysqldb://user:password@host/dbname')
    parser.add_argument('--table_names', nargs='?', action='store', help='数据库表，多个表以“,”分割')
    parser.add_argument('--sql_file', nargs='?', action='store', help='数据库建表语句的sql文件路径')
    parser.add_argument('--output', nargs='?', action='store', default=None, help='输出文件名，默认为数据库表名+meta.yml')
    args = parser.parse_args()

    if args.type == 'table_name' and (not args.connect or not args.table_names):
        print('You must supply a connect and table_names\n')
        parser.print_help()
        exit(0)

    if args.type == 'table_statement' and not args.sql_file:
        print('You must supply a sql_file\n')
        parser.print_help()
        exit(0)

    return args


def main():
    args = parse_args()
    start(**args.__dict__)


if __name__ == '__main__':
    main()



from faker.providers import BaseProvider
import random
import uuid
from dbfaker.common.logger import log
import os
import re
import json
from dbfaker.common.setting import check_path
from dbfaker.common.database import Database
import pypinyin


class Provider(BaseProvider):
    def __init__(self, generator, connect=None):
        super().__init__(generator)
        self.log = log
        self.db = None
        if connect:
            self.db = Database(connect)

    def hans2pinyin(self, hans, style='A'):
        """
        汉字转拼音
        :param hans: 汉字
        :param style: 返回首字母还是全拼， A：全拼； F：首字母
        :return:
        """

        if style.upper() == 'F':
            return ''.join(pypinyin.lazy_pinyin(hans=hans, style=pypinyin.Style.FIRST_LETTER))
        else:
            return ''.join(pypinyin.lazy_pinyin(hans=hans))

    def eq(self, value=None):
        """
        原样返回
        """
        return value

    def choice(self, value):
        if isinstance(value, str):
            try:
                value = json.loads(value)
            except json.decoder.JSONDecodeError:
                pass
        return random.choice(value)

    def choice_file(self, path, file_match=None, recursion=False, number=1):
        """
        指定目录寻找匹配规则的文件
        :param path:
        :param file_match:
        :param recursion:
        :return:
        """
        file_list = list(os.walk(check_path(path), ))
        if not file_list:
            return
        if not recursion:
            file_list = file_list[0][-1]
            result = [os.path.abspath(s.string) for s in [re.match(file_match, x) for x in file_list] if s]
        else:
            result = []
            for k in list(zip(*file_list))[-1]:
                result += k
            result = [os.path.abspath(s.string) for s in [re.match(file_match, x) for x in result] if s]
        if len(result) > number:
            result = random.sample(result, number)
        return result

    def uuid(self, underline=True):
        """
        返回遗传32位随机字符串
        """
        uid = str(uuid.uuid4())
        if underline:
            uid = uid.replace("-", '')
        self.log.d('生成uuid： {}'.format(uid))
        return uid

    def from_db(self, sql, choice_method='random', key=None):
        """
        从数据库中查数据
        :param sql: 查询语句
        :param choice_method: 对返回多个值时的处理方式，random： 随机取一个， first：取第一个，其它：返回整个list
        :param key: 当key存在时，将从返回数据键值对中取键为“key”的值
        :param tag:
        :return:
        """
        if not self.db:
            self.log.w('未指定数据库连接引擎，无法从数据库查询数据')
            return
        result = self.db.select3(sql)
        if not result:
            return
        if key:
            result = [i[key] for i in result]
        if choice_method == 'random':
            result = random.choice(result)
        elif choice_method == 'first':
            result = result[0]
        return result

    def from_db_yield(self, sql, key=None):
        """
        从数据库中查数据
        :param sql: 查询语句
        :param choice_method: 对返回多个值时的处理方式，random： 随机取一个， first：取第一个，其它：返回整个list
        :param key: 当key存在时，将从返回数据键值对中取键为“key”的值
        :return:
        """
        if not self.db:
            self.log.w('未指定数据库连接引擎，无法从数据库查询数据')
            return
        result = self.db.select3(sql)
        if key:
            result = [i[key] for i in result]
        for i in result:
            yield i

    def weights_randint(self, *args):
        """
        带权重的随机数，int型
        >> weights_randint({"weight": 0.8, "value": [1,2]}, {"weight": 0.2, "value": [3,4]})
        >> 1
        """
        result = []
        for d in args:
            _weights = d.get('weight')
            value = d.get('value')
            if isinstance(value, str):
                try:
                    value = json.loads(value)
                except json.decoder.JSONDecodeError:
                    pass

            n = 10
            while _weights < 1:
                _weights = int(_weights * n)
            for i in range(_weights):
                result.append(random.randint(*value))
        result = random.choice(result)
        return result

    def weights_randfloat(self, *args, round_num=2):
        """
        权重计算随机数float类型
        :param args:
        :param round_num:
        :return:
        """
        result = []
        for d in args:
            _weights = d.get('weight')
            value = d.get('value')
            if isinstance(value, str):
                try:
                    value = json.loads(value)
                except json.decoder.JSONDecodeError:
                    pass
            if d.get("round_num"):
                round_num = d.get("round_num")
            n = 10
            while _weights < 1:
                _weights = int(_weights * n)
            for i in range(_weights):
                result.append(round(random.uniform(*value), round_num))
        result = random.choice(result)
        return result

    def weights_choice(self, *args):
        """
        权重选择
        :param args:
        :return:
        """
        values = []
        for d in args:
            weights = d.get('weight')
            value = d.get('value')
            if isinstance(value, str):
                try:
                    value = json.loads(value)
                except json.decoder.JSONDecodeError:
                    pass
            if weights < 1:
                weights = int(weights * 100 / len(value))
            for i in value:
                values += [i] * weights
        result = random.choice(values)
        return result

    def multi_choice(self, value, splits="|", rand_number=None):
        """
        从指定的列表中随机选择多个，使用拼接字符拼起来返回
        :param value:
        :param splits:
        :return:
        """
        if isinstance(value, str):
            try:
                value = json.loads(value)
            except json.decoder.JSONDecodeError:
                pass
        if not rand_number or rand_number > len(value):
            rand_number = random.randint(0, len(value))
        return splits.join([str(i) for i in random.sample(value, rand_number)])

    def randint(self, value: list):
        """
        从给定的数值范围内随机返回一个int数值
        """
        if isinstance(value, str):
            try:
                value = json.loads(value)
            except json.decoder.JSONDecodeError:
                pass
        return random.randint(*value)

    def randfloat(self, value: list, round_num=2):
        """
        从给定的数值范围内随机返回一个float数值
        """
        if isinstance(value, str):
            try:
                value = json.loads(value)
            except json.decoder.JSONDecodeError:
                pass
        return round(random.uniform(*value), round_num)


    @staticmethod
    def order(value, out_of_order=False):
        """
        返回一个列表生成器对象，使用next()来调用
        """
        if out_of_order and isinstance(value, list):
            value = list(set(value))
        for i in value:
            yield i




# -*- encoding: utf-8 -*-
import pymysql
from dbfaker.common.drivers import load_sqlalchemy, load_conn
from dbfaker.common.db_helper import DBHelper
from sqlalchemy.engine import url

pymysql.install_as_MySQLdb()


class Database:
    db = None

    def __init__(self, db_session=None):
        self.__init_db(db_session)

    @classmethod
    def __init_db(cls, db_session):
        if not cls.db and db_session:
            cls.drivername, db_config = cls.__analyse_db_session(db_session)
            DBHelper.db_setting(**db_config)
            cls.db = DBHelper(cls.drivername)

    @staticmethod
    def __analyse_db_session(db_session):
        url_config = url.make_url(db_session)
        entrypoint = url_config._get_entrypoint()
        dialect_cls = entrypoint.get_dialect_cls(url_config)
        dbapi = dialect_cls.dbapi()

        db_config = {
            'creator': dbapi,
            'host': url_config.host,
            'user': url_config.username,
            'passwd': url_config.password,
            'db': url_config.database,
            'port': url_config.port
        }

        return url_config.drivername, db_config

    def select(self, table, where=None):
        """
        执行查询操作
        :param table:
        :param where:
        :return:
        """
        cur = self.db.cur
        if not where:
            sql = "select * from {}".format(table)
        else:
            sql = "select * from {table} ".format(table=table) + "where " + where
        cur.execute(sql)
        result = cur.fetchall()
        results = []
        col = [x[0] for x in cur.description]
        for i in result:
            data = dict(zip(col, i))
            results.append(data)
        return results

    def select2(self, table, where=None):
        """
        执行查询操作
        :param table:
        :param where:
        :return:
        """
        cur = self.db.cur
        if not where:
            sql = "select * from {}".format(table)
        else:
            ls = [(k, where[k]) for k in where if where[k]]
            sql = "select * from {} where ".format(table) + ' and '.join([i[0] + "=%r" % i[1] for i in ls])
        cur.execute(sql)
        result = cur.fetchall()
        results = []
        col = [x[0] for x in cur.description]
        for i in result:
            data = dict(zip(col, i))
            results.append(data)
        return results

    def select3(self, sql):
        """
        执行查询操作
        :param table:
        :param where:
        :return:
        """
        cur = self.db.cur
        cur.execute(sql)
        result = cur.fetchall()
        results = []
        col = [x[0] for x in cur.description]
        for i in result:
            data = dict(zip(col, i))
            results.append(data)
        return results

    def query(self, sql, raise_=True):
        """
        执行增删改操作
        :param sql:
        :return:
        """
        # print(sql)
        cur = self.db.cur
        try:
            cur.execute(sql)
            self.db.commit()
        except Exception as e:
            print(str(e) + "\n" + sql)
            self.db.rollback()
            if raise_:
                raise

    def insert(self, table, fields):
        """
        执行新增操作
        :param table:
        :param fields:
        :return:
        """
        keys = ', '.join(list(fields.keys()))
        values = tuple(fields.values())
        sql = "INSERT INTO {table}({keys}) value {values}".format(table=table, keys=keys, values=values)
        self.query(sql)

    def query_table(self, table_name):
        if self.drivername == 'sqlite':
            sql = "SELECT sql FROM sqlite_master WHERE type='table' AND name = '{}'".format(table_name)
            self.db.cur.execute(sql)
            result = self.db.cur.fetchall()
            return result[0][0]
        else:
            sql = 'show create table {table_name};'.format(table_name=table_name)
            self.db.cur.execute(sql)
            result = self.db.cur.fetchall()
            return result[0][1]

    def insert2(self, table, fields):
        """
        执行新增操作
        :param table:
        :param fields:
        :return:
        """
        ls = [(k, fields[k]) for k in fields if fields[k]]
        sql = "insert into {} set ".format(table) + ', '.join([i[0] + "=%r" % i[1] for i in ls])
        self.query(sql)

    def close(self):
        self.db.dispose()


if __name__ == '__main__':
    db = Database(db_session='mysql+mysqldb://root:mysql123@cpcs.homelabs.in/center')
    r = db.select3('select id, username from t_sys_user where id=1')
    print(r)
    r = db.select3('select id, username from t_sys_user where id=1')
    print(r)



import unittest
import os
import sqlite3


class MyTestCase(unittest.TestCase):
    os.chdir('..')

    def test_001(self):
        print(os.popen('dbfaker data/test.yml -o test.sql -n 100').read())
        s = list(os.walk('.'))
        self.assertIn('test.sql', s[0][-1])
        os.remove('test.sql')

    def test_002(self):
        NUMBER = 10  # 生成数据数量
        db = sqlite3.connect('data/test.db')
        cur = db.cursor()
        cur.execute('select count(1) from stu')
        start_num = cur.fetchall()[0][0]
        print(os.popen('dbfaker data/test.yml -i --connect sqlite:///data/test.db -n {}'.format(NUMBER)).read())
        cur.execute('select count(1) from stu')
        end_num = cur.fetchall()[0][0]
        self.assertEqual(end_num-start_num, NUMBER)

if __name__ == '__main__':
    unittest.main()



from faker.generator import Generator


class MGenerator(Generator):

    def add_provider(self, provider, *args, **kwargs):
        """
        使add_provider支持传参给provider对象
        """
        if isinstance(provider, type):
            provider = provider(self, *args, **kwargs)

        self.providers.insert(0, provider)

        for method_name in dir(provider):
            # skip 'private' method
            if method_name.startswith('_'):
                continue

            faker_function = getattr(provider, method_name)

            if callable(faker_function):
                # add all faker method to generator
                self.set_formatter(method_name, faker_function)



import os

SAMPLE_YAML_DATA = '''package:
  - datetime  #　导入额外的包，在jinja2模板中使用（下面有用到datetime包，所以要先导入）

env:
  id:
    engine: faker.uuid
    rule: null
  time_format:
    engine: faker.eq
    rule:
      value: "%Y-%m-%d %H:%M:%S"

tables:
- columns:
  - column: id
    comment: 数据主键id
    engine: faker.eq
    rule:
      value: '{{ env.id }}'  # 通过引用环境变量中的值
  - column: name
    comment: 姓名
    engine: faker.name
    rule: null
  - column: idcard
    comment: 身份证号
    engine: faker.ssn
    rule: null
  - column: age
    comment: 年龄
    engine: faker.eq
    rule:
      value: '{{ datetime.datetime.now().year - int(stu.idcard[6:10]) }}'  #　通过jinja２模板直接计算
  - column: sex
    comment: 性别
    engine: faker.eq
    rule:
      value: '{{ "man" if int(stu.idcard[-2]) % 2==1 else "female" }}'  #　通过jinja２模板直接计算
  comment: ''
  table: stu
- columns:
  - column: id
    comment: 数据主键id
    engine: faker.uuid
    rule: null
  - column: stu_id
    comment: 数据主键id
    engine: faker.eq
    rule:
      value: '{{ stu.id }}'  # 通过其他表中的值
  - column: course_name
    comment: 课程名称
    engine: faker.choice # 通过内置方法从列表中随机取一个值
    rule:
      value: [数学,语文,英语,化学,地理]
  - column: course_time
    comment: 上课时间
    engine: faker.now  # 通过内置方法获取当前时间，并按照指定格式返回
    rule:
      format: "{{ env.time_format }}"
  comment: '课程信息 '
  table: course

extraction:  # 从已生成的数据中提取字段
  stu_id:
    value: '{{ stu.id }}'
    default: null
  course_id:
    value: '{{ course.id }}'
  stu_name:
    value: '{{ stu.name }}'
    default: '测试用户'

'''

SAMPLE_FUNCTION = '''from faker.providers import BaseProvider
import hashlib


class MProvider(BaseProvider):
    def md5(self, value: bytes):
        if isinstance(value, str):
            value = value.encode()
        return hashlib.md5(value).hexdigest()
'''


README = '''# 目录结构说明
```
.
├── data    # yaml文件存放位置
├── log     # 执行日志
└── script  # 自定义方法的Python文件存放位置；在此路径中的所有py结尾的文件中的继承faker.BaseProvider类的子类会被动态加载到faker执行方法中。
```
'''

def scaffold(project_name='dbfaker-project', path=None):
    '''
    初始化dbfaker项目
    '''
    if not path:
        path = os.path.curdir
    if not os.path.isdir(path):
        raise OSError('目录错误')
    cur_dir = os.path.join(path, project_name)
    os.mkdir(cur_dir) if not os.path.exists(cur_dir) else None
    os.chdir(cur_dir)
    os.mkdir('data') if not os.path.exists('data') else None
    os.mkdir('log') if not os.path.exists('log') else None
    os.mkdir('script') if not os.path.exists('script') else None
    with open(os.path.join('data', 'sample.yml'), 'w', encoding='utf-8') as f:
        f.write(SAMPLE_YAML_DATA)
    with open(os.path.join('script', 'sample.py'), 'w', encoding='utf-8') as f:
        f.write(SAMPLE_FUNCTION)
    with open(os.path.join('readme.md'), 'w', encoding='utf-8') as f:
        f.write(README)
    print('项目初始化完成')



import unittest
from dbfaker.utils.faker_tool import Provider
from faker import Faker
from dbfaker.utils.generator import MGenerator

mfaker = Faker(locale='zh_CN', generator=MGenerator(locale='zh_CN'))

# 测试连接地址为内部地址，外部访问将会失败。
mfaker.add_provider(Provider, connect='mysql+mysqldb://pdmsadmin:system001@cpcs.homelabs.in/pdms_hospital')

class MfakerTestCase(unittest.TestCase):
    mfaker = mfaker
    def test_uuid_1(self):
        uid = self.mfaker.uuid(underline=True)
        self.assertEqual(len(uid), 32)

    def test_uuid_2(self):
        uid = self.mfaker.uuid(underline=False)
        self.assertEqual(len(uid), 36)

    def test_uuid_3(self):
        uid = self.mfaker.uuid(underline=False)
        self.assertEqual(len(uid), 36)

    def test_from_db(self):
        db_result = self.mfaker.from_db(
            sql='select id from device_info where class_id=(select id from device_classify where serial=\'HL-ECG\')',
            choice_method='first', key='id')
        self.assertEqual(bool(db_result), True)

    def test_choice_file(self):
        self.mfaker.choice_file(path='.', file_match='.*?\.(png|jpg|bmp|PNG|JPG|BMP|py)$', recursion=True)
        self.mfaker.choice_file(path='../../', file_match='.*?\.(png|jpg|bmp|PNG|JPG|BMP|py)$', recursion=True, number=3)
        self.mfaker.choice_file(path='../../', file_match='.*?\.(png|jpg|bmp|PNG|JPG|BMP|py)$', recursion=False)

    def test_choice(self):
        data = [1,2,3,4,5,6,7,8]
        r = self.mfaker.choice(data)
        self.assertIn(r, data)

    def test_from_db_yield(self):
        result = self.mfaker.from_db_yield(
            sql="select id from report_detail_properties where name in ('QT', 'PR', 'QTC', 'RV5', 'HR', 'QRSaxis', 'R+S', 'QRS', 'P', 'SV1')",
            key='id')
        while 1:
            try:
                next(result)
            except StopIteration:
                break
        # self.assertEqual(len(r), 32)

if __name__ == '__main__':
    unittest.main()



import unittest
import os

class MyTestCase(unittest.TestCase):
    os.chdir('..')
    def test_from_file(self):
        os.popen('table2yml table_statement --sql_file "data/test.sql" --output stu_meta.yml').read()
        s = list(os.walk('.'))
        self.assertIn('stu_meta.yml', s[0][-1])
        os.remove('stu_meta.yml')

    def test_from_db(self):
        os.popen('table2yml table_name --table_names "stu,course,choice_course" --connect sqlite:///data/test.db --output stu_meta.yml').read()
        s = list(os.walk('.'))
        self.assertIn('stu_meta.yml', s[0][-1])
        os.remove('stu_meta.yml')



if __name__ == '__main__':
    unittest.main()



# -*- encoding: utf-8 -*-
import datetime
import logging
import os
from logging import handlers

from dbfaker.common.tools import check_path

__all__ = ['Logger', 'log']


class Logger:
    LOG_FORMAT = '%(asctime)s [%(levelname)s]   [%(filename_ca)s] [line:%(lineno_ca)d]: %(message)s'
    DATE_FORMAT = '%Y/%m/%d %H:%M:%S %p'

    def __init__(self, name, path=None):
        self._logger = logging.Logger(name)
        self._logger.setLevel(logging.DEBUG)
        fmt = logging.Formatter(Logger.LOG_FORMAT)
        if path:
            path = check_path(path)
            if not os.path.exists(os.path.dirname(path)):
                os.mkdir(os.path.dirname(path))
            # print('log into file [%s].' % os.path.abspath(path))
            file_handler = handlers.TimedRotatingFileHandler(path, when='midnight', interval=1, backupCount=10,
                                                             encoding='utf-8', atTime=datetime.time(0, 0, 0, 0))
            file_handler.setFormatter(fmt)
            file_handler.setLevel(logging.DEBUG)
            self._logger.addHandler(file_handler)
        # stream_handler = logging.StreamHandler()
        # stream_handler.setLevel(logging.DEBUG)
        # stream_handler.setFormatter(fmt)
        # self._logger.addHandler(stream_handler)

    def i(self, msg, *args, **kwargs):
        self._logger.info(msg, *args, **self.wrap_stack(**kwargs))

    def d(self, msg, *args, **kwargs):
        self._logger.debug(msg, *args, **self.wrap_stack(**kwargs))

    def w(self, msg, *args, **kwargs):
        self._logger.warning(msg, *args, **self.wrap_stack(**kwargs))

    def e(self, msg, *args, **kwargs):
        self._logger.error(msg, *args, **self.wrap_stack(**kwargs))

    def exception(self, msg, *args, exc_info=True, **kwargs):
        self._logger.exception(msg, *args, exc_info=exc_info, **self.wrap_stack(**kwargs))

    def c(self, msg, *args, **kwargs):
        self._logger.critical(msg, *args, **self.wrap_stack(**kwargs))

    def wrap_stack(self, **kwargs):
        extra = kwargs['extra'] if 'extra' in kwargs else {}
        caller = self._logger.findCaller(stack_info=False)
        cwd = os.getcwd().replace('\\', '/')
        caller_file = caller[0].replace(os.getcwd(), '')
        caller_file = caller_file.replace(cwd, '')
        if caller_file.startswith('/') or caller_file.startswith('\\'):
            caller_file = caller_file[1:]
        extra['filename_ca'] = caller_file
        extra['lineno_ca'] = caller[1]
        kwargs['extra'] = extra
        return kwargs


log = Logger('dbfaker', path=r'log/dbfaker.log')



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker


def load_sqlalchemy(connet):
    """
    返回db session
    :param connet:
    :return:
    """
    engine = create_engine(connet, pool_recycle=1800)
    DBSession = sessionmaker(engine)
    session = DBSession()
    return session


def load_conn(connet):
    engine = create_engine(connet, pool_recycle=1800)
    return engine.connect()



# -*- encoding: utf-8 -*-
from dbfaker.utils.gendata import DataGenerator
import sys
import argparse
from dbfaker.utils.constant import __version__
from dbfaker.utils.faker_date_time import Provider as DateTimeProvider
from dbfaker.utils.faker_tool import Provider as ToolProvider
from faker import Faker
from dbfaker.utils.generator import MGenerator
from tqdm import tqdm
from dbfaker.utils.init_project import scaffold
import os
from dbfaker.common.logger import log
import importlib

faker = Faker(locale='zh_CN', generator=MGenerator(locale='zh_CN'))


def loop(meta_file, number=1, insert=False, connect=None, output=None, _print=True, **kwargs):
    handler = DataGenerator(faker, meta_file, connect, )
    handler.import_package()
    for i in tqdm(range(number), unit='组'):
        sys.stdout.flush()
        handler.start()
    if insert:
        handler.insert2db()
    if output:
        handler.save(output=output)
    if _print:
        [print(i) for i in handler.sqls]

    print('执行完成，共生成{number}组数据'.format(number=number))


def parse_args():
    if '--version' in sys.argv:
        print(__version__)
        exit(0)

    parser = argparse.ArgumentParser(
        description='通过ｙｍｌ格式的描述文件来生成数据')
    parser.add_argument('meta_file', nargs='?', action='store',
                        help='yml文件所在路径')
    parser.add_argument('-n', '--number', nargs='?', action='store', default=1,
                        help='生成数据数量', type=int)
    parser.add_argument('-i', '--insert', action='store_true', help='是否插入到数据库')
    parser.add_argument('-c', '--connect', nargs='?', action='store',
                        help='数据库连接语法，例如：mysql+mysqldb://pdmsadmin:system001@cpcs.homelabs.in/pdms_hospital')

    parser.add_argument('-o', '--output', nargs='?', action='store',
                        help='指定文件名，输出内容到文件')
    parser.add_argument('-p', '--_print', action='store_true', help='是否打印到控制台')
    parser.add_argument('--project_name', action='store', help='初始化项目时的项目名称', default='dbfaker-project')
    args = parser.parse_args()
    if not args.meta_file:
        print('You must supply meta_file\n')
        parser.print_help()
        exit(0)

    if args.meta_file == 'init':
        scaffold(project_name=args.project_name)
        exit(0)

    if args.insert and not args.connect:
        print('You must supply connect\n')
        parser.print_help()
        exit(0)

    return args


def import_module(module_path='script'):
    if not os.path.exists(module_path):
        return
    pys = [x for x in list(os.walk(module_path))[0][-1] if x.endswith(".py")]
    modules = []
    for py in pys:
        module = importlib.import_module('{}.{}'.format(module_path, py.split(".")[0]))
        for class_name in dir(module):
            provider = getattr(module, class_name)
            if isinstance(provider, type):
                modules.append(provider)
    return modules


def run():
    cur_dir = os.path.abspath(os.curdir)
    sys.path.append(cur_dir)
    args = parse_args()
    faker.add_provider(DateTimeProvider, offset=0)
    faker.add_provider(ToolProvider, connect=args.__dict__.get("connect"))
    modules = import_module()
    if modules:
        for module in modules:
            faker.add_provider(module)
    loop(**args.__dict__)


if __name__ == '__main__':
    run()



# coding=utf-8
import pymysql
from dbutils.pooled_db import PooledDB
from dbutils.persistent_db import PersistentDB

# 数据库配置信息
db_info = {}


class DBHelper:
    """数据库连接助手"""
    __POOL = None

    def __init__(self, drivername='pymysql'):
        self.conn = self.__get_conn(drivername)
        self.cur = self.conn.cursor()

    @staticmethod
    def __get_conn(drivername):
        """获取连接"""
        global __POOL
        if drivername == 'sqlite':
            engine = PersistentDB
            db = db_info['db']
            creator = db_info['creator']
            db_info.clear()
            db_info['database'] = db
            db_info['creator'] = creator
        else:
            engine = PooledDB

        if DBHelper.__POOL is None:
            __POOL = engine(**db_info)
        return __POOL.connection()

    @staticmethod
    def db_setting(db, creator=pymysql, user='root', passwd='123456', host='localhost', port=3306):
        """设置数据库配置信息"""
        db_info.update({
            'db': db,
            'creator': creator,
            'user': user,
            'passwd': passwd,
            'host': host,
            'port': port,
            'autocommit': 1
        })

    def execute(self, sql):
        """执行SQL语句"""
        effect_num = self.cur.execute(sql)
        return effect_num

    def commit(self):
        """提交操作"""
        self.conn.commit()

    def rollback(self):
        """回滚操作"""
        self.conn.rollback()

    def dispose(self):
        """释放连接"""
        self.cur.close()
        self.conn.close()



#! /usr/bin/env python
# -*- coding: utf-8 -*_
# Author: guolong<565169745@qq.com>
from distutils.core import setup
import setuptools
import os
import codecs
import re

DESCRIPTION = ''
AUTHOR = ''
URL = ''
VERSION = ''
EMAIL_URL = ''

here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    return codecs.open(os.path.join(here, *parts), encoding='utf-8').read()


def find_config(*file_paths):
    config_file = read(*file_paths)
    global VERSION, DESCRIPTION, URL, AUTHOR, EMAIL_URL
    VERSION = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                        config_file, re.M).group(1)
    DESCRIPTION = re.search(r"^__description__ = ['\"]([^'\"]*)['\"]",
                            config_file, re.M).group(1)
    AUTHOR = re.search(r"^__author__ = ['\"]([^'\"]*)['\"]",
                       config_file, re.M).group(1)
    URL = re.search(r"^__url__ = ['\"]([^'\"]*)['\"]",
                    config_file, re.M).group(1)
    EMAIL_URL = re.search(r"^__author_email__ = ['\"]([^'\"]*)['\"]",
                    config_file, re.M).group(1)


with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

find_config('dbfaker/utils/constant.py')

setup(
    name='dbfaker',
    version=VERSION,
    description=DESCRIPTION,
    author=AUTHOR,
    author_email=EMAIL_URL,
    url=URL,
    packages=setuptools.find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    entry_points={'console_scripts': [
        'dbfaker = dbfaker.cli:run',
        'table2yml = dbfaker.table2yml:main'
    ]},
    install_requires=[
        'PyMySQL',
        'PyYAML',
        'pypinyin',
        'Faker',
        'Jinja2',
        'SQLAlchemy',
        'ply',
        'tqdm',
        'dbutils'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries'
    ],
    zip_safe=True,
)



from dbfaker.common.setting import get_yaml
import jinja2
import json
import copy
from dbfaker.common.logger import log
from dbfaker.common.database import Database
from tqdm import tqdm
import sys


class DataGenerator:
    def __init__(self, faker, meta, connect=None):
        self.all_package = {}
        self.pre_data = {}
        self.result_data = {}
        self.extraction_data = {}
        self.log = log
        self.error_fields = {}
        self.faker = faker
        self.sqls = []
        self.db = Database(connect) if connect else None
        self.meta = get_yaml(meta)

    def import_package(self):
        self._import_package()

    def start(self):
        self.result_data = {}
        self.extraction_data = {}
        self._env()
        self.mock_data()
        self.extraction()
        self._error_field()
        self.dict2sql()
        return self

    def save(self, output=None):
        if not output:
            return
        f = open(output, 'w')
        f.write('\n'.join(self.sqls))
        f.close()

    def _import_package(self):
        """
        动态导包
        """
        if isinstance(__builtins__, dict):
            self.all_package.update(__builtins__)
        else:
            self.all_package.update(__builtins__.__dict__)
        packages = self.meta.get('package')
        if not packages:
            return
        for i in packages:
            try:
                self.all_package[i] = __import__(i)
            except ModuleNotFoundError:
                pass

    def _env(self):
        """
        环境变量预处理
        :param condition:
        :return:
        """
        self.pre_data['env'] = {}
        env = self.meta.get('env')
        if not env:
            return
        for i, j in env.items():
            engine = j.get("engine")
            rule = j.get("rule")
            rule = json.dumps(rule)
            result = self._gen_data(engine, rule)
            self.pre_data['env'][i] = result

    def _field(self, columns):
        result = {}
        pre_data = {}
        error_field = []
        table_name = columns.get("table")
        pre_data[table_name] = {}
        for field in columns.get('columns'):
            field_column = field.get('column')
            field_engine = field.get('engine')
            field_rule = field.get("rule")
            if field_rule:
                _rule = json.dumps(field.get("rule"))
                _rule = _rule.replace('\\\"', "'")
                try:
                    field_rule = json.loads(self._template_render(_rule))
                except jinja2.exceptions.UndefinedError as e:
                    error_field.append(field)
                    continue
            _r = self._gen_data(field_engine, field_rule)
            result[field_column] = _r
            pre_data[table_name][field_column] = _r
            if table_name not in self.pre_data:
                self.pre_data[table_name] = {}
            self.pre_data[table_name].update(pre_data[table_name])
            if error_field:
                # 记录表中出错的字段，在所有字段都生成后再次生成，解决因前面字段调用未生成字段时报错问题
                self.error_fields.update({'columns': error_field, 'table': table_name})
        return result

    def _more_field(self, columns, max_cnt=1):
        results = []
        result = {}
        table_name = columns.get("table")
        n = 0
        while n < max_cnt:
            try:
                for field in columns.get('columns'):
                    field_column = field.get('column')
                    field_engine = field.get('engine')
                    field_rule = json.loads(self._template_render(json.dumps(field.get("rule"))))
                    _r = self._gen_data(field_engine, field_rule)
                    result[field_column] = _r
            except jinja2.exceptions.UndefinedError as e:
                break
            results.append(copy.deepcopy(result))
            n += 1
        self.pre_data[table_name] = results
        return results

    def _error_field(self):
        """
        异常字段重新处理，解决引用的字段还未生成时造成的引用失败的问题
        """
        if not self.error_fields:
            return
        self.mock_data([self.error_fields])

    def _template_render(self, s, env=None):
        """
        jinja2模板渲染
        """
        tp = jinja2.Template(s, undefined=jinja2.StrictUndefined)
        if isinstance(env, dict):
            self.all_package.update(env)
        tp.globals.update(self.all_package)
        r = tp.render(**self.pre_data)
        return r

    def _gen_data(self, engine: str, rule):
        if isinstance(rule, str):
            rule = json.loads(self._template_render(rule))
        faker = self.faker
        if not engine:
            return
        if '.' not in engine:
            engine = "faker.{engine}".format(engine=engine)
        if isinstance(rule, list):
            r = eval("{engine}(*{rule})".format(engine=engine, rule=rule))
        elif isinstance(rule, dict):
            r = eval("{engine}(**{rule})".format(engine=engine, rule=rule))
        elif rule is None:
            r = eval("{engine}()".format(engine=engine))
        else:
            raise Exception('rule type must be dictionary or list！')
        return r

    def mock_data(self, fields=None):

        if not fields:
            fields = self.meta.get('tables')
        for columns in fields:
            table_name = columns.get("table")
            more = columns.get('more')
            if not more:
                d = self._field(columns)
            else:
                max_cnt = more.get("max_number", 1)
                if isinstance(max_cnt, str):
                    max_cnt = int(self._template_render(max_cnt))
                d = self._more_field(columns=columns, max_cnt=max_cnt)
            if table_name not in self.result_data and not more:
                self.result_data[table_name] = {}
                self.result_data[table_name].update(d)
            elif table_name not in self.result_data:
                self.result_data[table_name] = d

        return self.result_data

    def extraction(self):
        extraction_metas = self.meta.get('extraction')
        if not extraction_metas:
            return self.extraction_data

        for key, values in extraction_metas.items():
            value = {'value': self._template_render(values.get('value', ''))}
            r_value = self._gen_data('eq', value)
            default = values.pop("default") if 'default' in values else None
            if not r_value:
                r_value = default
            self.extraction_data[key] = r_value
        return self.extraction_data

    def dict2sql(self, datas=None):
        if not datas and isinstance(self.result_data, dict):
            datas = copy.deepcopy(self.result_data)

        def gen_sql(table_name, data):
            """
            :param table_name: 表名称
            :param data: 字典对象 key为字段(要与数据库字段一样), value为插入值
            :return: 拼接好的sql语句
            """

            ls = [('`' + k + '`', v) for k, v in data.items() if v is not None]
            sentence = 'INSERT INTO `%s` (' % table_name + ','.join([i[0] for i in ls]) + \
                       ') VALUES (' + ','.join(repr(i[1]) for i in ls) + ');'
            return sentence

        for table, fields in datas.items():
            if isinstance(fields, list):
                for i in fields:
                    sql = gen_sql(table, i)
                    self.sqls.append(sql)
            elif isinstance(fields, dict):
                sql = gen_sql(table, fields)
                self.sqls.append(sql)
        return self.sqls

    def insert2db(self):
        self.log.i('开始插入数据至数据库')
        print('开始插入数据至数据库')
        if not self.db:
            self.log.w('未指定数据库连接引擎，无法插入到数据库...')
            return
        for sql in tqdm(self.sqls, unit='条'):
            sys.stdout.flush()
            self.db.query(sql)
        self.log.i('在数据库中插入了{}条数据'.format(len(self.sqls)))
        print('在数据库中插入了{}条数据'.format(len(self.sqls)))




import os

import yaml

from dbfaker.common.logger import log
from dbfaker.common.tools import check_path

__all__ = ['get_yaml', 'check_valid', 'update_settings']


def check_valid(key, value):
    if not value:
        log.e('[%s] Not Configured.Please Config in settings.yaml.' % key)
        raise ValueError
    return value


def get_yaml(yaml_file=None):
    yaml_file = check_path(yaml_file)
    with open(yaml_file, 'r', encoding='utf-8') as f:
        return yaml.safe_load(f.read())


def update_settings(setting_data):
    register_setting = setting_data.get("Register")
    _global_setting = setting_data.get("global")
    for register_module in register_setting:
        for g in _global_setting:
            if g not in register_setting[register_module]:
                register_setting[register_module][g] = _global_setting[g]

    return setting_data



# coding:utf8
import os, sys
import re
import ply.yacc as yacc
from dbfaker.nsqlparse.mysql_create_table_lex import tokens
import json


def p_error(p):
    print('-------------', p.lineno, dir(p))
    raise SyntaxError(p)


################## 入口

def p_start(p):
    '''
    start : createTable
    '''
    p[0] = [p[1]]


def p_start_empty(p):
    '''
    start : ';'
    '''
    p[0] = []


def p_start_mul(p):
    '''
    start : start ';' createTable
    '''
    p[0] = p[1] + [p[3]]


def p_start_empty1(p):
    '''
    start : start ';'
    '''
    p[0] = p[1]


################## 创建表

def p_createTable_columns(p):
    '''
    createTable : CREATE TABLE tableName '(' tableColumns ')'
    '''
    p[0] = {**p[3], **{'columns': p[5]}}


def p_createTable_keys(p):
    '''
    createTable : CREATE TABLE tableName '(' tableColumns ',' tableKeys ')'
    '''
    p[0] = {**p[3], **{'columns': p[5]}, **{'keys': p[7]}}


def p_createTable_attribute_default_charset(p):
    '''
    createTable : createTable DEFAULT CHARSET '=' iKEYWORD
    '''
    p[0] = {**p[1], **{'default_charset': p[5].lower()}}


def p_createTable_attribute_engine(p):
    '''
    createTable : createTable ENGINE '=' iKEYWORD
    '''
    p[0] = {**p[1], **{'engine': p[4].lower()}}


def p_createTable_attribute_auto_increment(p):
    '''
    createTable : createTable AUTO_INCREMENT '=' iINTEGER
    '''
    p[0] = {**p[1], **{'auto_increment': int(p[4])}}


def p_createTable_attribute_comment(p):
    '''
    createTable : createTable COMMENT '=' str
    '''
    p[0] = {**p[1], **{'comment': p[4]}}


def p_createTable_attribute_collate(p):
    '''
    createTable : createTable COLLATE '=' iKEYWORD
    '''
    p[0] = {**p[1], **{'collate': p[4].lower()}}


def p_createTable_attribute_row_format(p):
    '''
    createTable : createTable ROW_FORMAT '=' iKEYWORD
    '''
    p[0] = {**p[1], **{'row_format': p[4].lower()}}


################## 表索引定义

def p_tableKeys_item(p):
    '''
    tableKeys : tableKey
    '''
    p[0] = [p[1]]


def p_tableKeys_list(p):
    '''
    tableKeys : tableKeys ',' tableKey
    '''
    p[0] = p[1] + [p[3]]


def p_tableKey(p):
    '''
    tableKey : KEY columnName '(' columnNames ')'
    '''
    p[0] = {'columns': p[4], 'name': p[2]['column'], 'type': 'key'}


def p_tableKey_primary(p):
    '''
    tableKey : PRIMARY KEY '(' columnNames ')'
    '''
    p[0] = {'columns': p[4], 'type': 'primary'}


def p_tableKey_unique(p):
    '''
    tableKey : UNIQUE KEY columnName '(' columnNames ')'
    '''
    p[0] = {'columns': p[5], 'name': p[3]['column'], 'type': 'unique'}


def p_tableKey_unique_comment(p):
    '''
    tableKey : tableKey COMMENT columnName
    '''

def p_tableKey_using(p):
    '''
    tableKey : tableKey USING iKEYWORD
    '''
    p[0] = {**p[1], 'using': p[3].upper()}


################## 表字段定义

def p_tableColumns_item(p):
    '''
    tableColumns : tableColumn
    '''
    p[0] = [p[1]]


def p_tableColumns_list(p):
    '''
    tableColumns : tableColumns ',' tableColumn
    '''
    p[0] = p[1] + [p[3]]


def p_tableColumn(p):
    '''
    tableColumn : columnName dataType
    '''
    p[0] = {**p[1], **p[2]}


def p_tableColumn_attribute_notnull(p):
    '''
    tableColumn : tableColumn NOT NULL
    '''
    p[0] = {**p[1], **{'not_null': True}}


def p_tableColumn_attribute_null(p):
    '''
    tableColumn : tableColumn NULL
    '''
    p[0] = {**p[1], **{'not_null': False}}


def p_tableColumn_attribute_comment(p):
    '''
    tableColumn : tableColumn COMMENT str
    '''
    p[0] = {**p[1], **{'comment': p[3]}}


def p_tableColumn_attribute_auto_increment(p):
    '''
    tableColumn : tableColumn AUTO_INCREMENT
    '''
    p[0] = {**p[1], **{'auto_increment': True}}


def p_tableColumn_attribute_collate(p):
    '''
    tableColumn : tableColumn COLLATE iKEYWORD
    '''
    p[0] = {**p[1], **{'collate': p[3].lower()}}


def p_tableColumn_attribute_character_set(p):
    '''
    tableColumn : tableColumn CHARACTER SET iKEYWORD
    '''
    p[0] = {**p[1], **{'character_set': p[3].lower()}}


### 字段默认值

def p_tableColumn_attribute_default_int(p):
    '''
    tableColumn : tableColumn DEFAULT iINTEGER
    '''
    p[0] = {**p[1], **{'default': int(p[3])}}


def p_tableColumn_attribute_default_float(p):
    '''
    tableColumn : tableColumn DEFAULT iFLOAT
    '''
    p[0] = {**p[1], **{'default': float(p[3])}}


def p_tableColumn_attribute_default_str(p):
    '''
    tableColumn : tableColumn DEFAULT str
    '''
    p[0] = {**p[1], **{'default': '\'' + p[3] + '\''}}


def p_tableColumn_attribute_default_keyword(p):
    '''
    tableColumn : tableColumn DEFAULT iKEYWORD
                | tableColumn DEFAULT NULL
    '''
    p[0] = {**p[1], **{'default': p[3].upper()}}

def p_tableColumn_attribute_default_current_timestamp(p):
    '''
    tableColumn : tableColumn DEFAULT CURRENT_TIMESTAMP '(' iINTEGER ')'
                | tableColumn DEFAULT CURRENT_TIMESTAMP
    '''
    p[0] = {**p[1], **{'default': 'CURRENT_TIMESTAMP({})'.format(p[5])}}

def p_tableColumn_attribute_default_update_current_timestamp(p):
    '''
    tableColumn : tableColumn ON UPDATE CURRENT_TIMESTAMP '(' iINTEGER ')'
                | tableColumn ON UPDATE CURRENT_TIMESTAMP
    '''
    p[0] = {**p[1]}


### 字段数据类型定义

def p_dataType_normal(p):
    '''
    dataType : iKEYWORD
    '''
    p[0] = {'type': p[1]}


def p_dataType_len(p):
    '''
    dataType : iKEYWORD '(' iINTEGER ')'
    '''
    p[0] = {'type': p[1], 'length': p[3]}




def p_dataType_decimal(p):
    '''
    dataType : iKEYWORD '(' iINTEGER ',' iINTEGER ')'
    '''
    p[0] = {'type': p[1], 'length': p[3] + ',' + p[5]}


def p_dataType_enum(p):
    '''
    dataType : iKEYWORD '(' dataTypeEnum ')'
    '''
    p[0] = {'type': p[1], 'length': p[3]}


def p_dataTypeEnum_item(p):
    '''
    dataTypeEnum : str
    '''
    p[0] = [p[1]]


def p_dataTypeEnum_list(p):
    '''
    dataTypeEnum : dataTypeEnum ',' str
    '''
    p[0] = p[1] + [p[3]]


def p_dataType_unsigned(p):
    '''
    dataType : dataType UNSIGNED
    '''
    p[0] = {**p[1], **{'unsigned': True}}


################## 数据库、数据表、字段名称定义

def p_name_char(p):
    '''
    name : iCHAR
    '''
    p[0] = p[1]


def p_name_escape(p):
    '''
    name : iESCAPE
    '''
    p[0] = '`'


def p_name_str(p):
    '''
    name : name iCHAR
    '''
    p[0] = p[1] + p[2]


def p_name_str_escape(p):
    '''
    name : name iESCAPE
    '''
    p[0] = p[1] + '`'


def p_name_safe(p):
    '''
    name : iLBRACE name iRBRACE
    '''
    p[0] = p[2]


def p_databaseName(p):
    '''
    databaseName : name
    '''
    p[0] = p[1]


def p_tableName(p):
    '''
    tableName : name
    '''
    p[0] = {'table': p[1]}


def p_tableName_for_db(p):
    '''
    tableName : databaseName '.' name
    '''
    p[0] = {'database': p[1], 'table': p[3]}


def p_columnName(p):
    '''
    columnName : name
    '''
    p[0] = {'column': p[1]}


def p_columnName_for_table(p):
    '''
    columnName : tableName '.' name
    '''
    p[0] = {**p[1], **{'column': p[3]}}


def p_columnName_len(p):
    '''
    columnName : columnName '(' iINTEGER ')'
    '''
    p[0] = {**p[1], **{'length': int(p[3])}}


def p_columnNames_item(p):
    '''
    columnNames : columnName
    '''
    p[0] = [p[1]]


def p_columnNames_list(p):
    '''
    columnNames : columnNames ',' columnName
    '''
    p[0] = p[1] + [p[3]]


################## 字符串定义

def p_str_char(p):
    '''
    str : iCHAR
    '''
    p[0] = p[1]


def p_str_escape(p):
    '''
    str : iESCAPE
    '''
    p[0] = '\''


def p_str_str(p):
    '''
    str : str iCHAR
    '''
    p[0] = p[1] + p[2]


def p_str_str_escape(p):
    '''
    str : str iESCAPE
    '''
    p[0] = p[1] + '\''


def p_str(p):
    '''
    str : iLBRACE str iRBRACE
    '''
    p[0] = p[2]


def p_str_empty(p):
    '''
    str : iLBRACE iRBRACE
    '''
    p[0] = ''


parser = yacc.yacc(start='start', debug=False, write_tables=False)

if __name__ == '__main__':
    d = '''CREATE TABLE `stu` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT COMMENT '自增id',
  `name` varchar(20) NOT NULL COMMENT '学生名字',
  `school` varchar(20) NOT NULL COMMENT '学校名字',
  `nickname` varchar(20) NOT NULL COMMENT '学生小名',
  `age` int(11) NOT NULL COMMENT '学生年龄',
  `class_num` int(11) NOT NULL COMMENT '班级人数',
  `score` decimal(4,2) NOT NULL COMMENT '成绩',
  `phone` int(20) NOT NULL COMMENT '电话号码',
  `email` varchar(64) DEFAULT NULL COMMENT '家庭网络邮箱',
  `ip` varchar(32) DEFAULT NULL COMMENT 'IP地址',
  `address` text COMMENT '家庭地址',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
	
	'''
    r = parser.parse(d)
    print(r)
